/* 
 * Copyright (C) 1997-2002 Laird Breyer
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Author:   Laird Breyer (laird@lbreyer.com)
 */

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.util.*;

class AlgoControlPanel extends Panel implements ActionListener, ItemListener {
    static final int RWM = 0;
    static final int LAN = 1;
    static final int LFM = 6;
    static final int TRW = 8;
    static final int UI = 2;
    static final int UUI = 7;
    static final int GOR = 3;
    static final int GAR = 4;
    static final int HYB = 5;
    static final int URWM = 9;

    static final String RWMSTR = "Gaussian Random Walk Metropolis";
    static final String URWMSTR = "Uniform Random Walk Metropolis";
    static final String LANSTR = "Langevin";
    static final String LFMSTR = "Levy Flight Metropolis";
    static final String TRWSTR = "Student t Random Walk";
    static final String UISTR = "U[0,1]-Independence Sampler";
    static final String UUISTR = "General Independence Sampler";
    static final String GORSTR = "Gareth's mode hopper";
    static final String GARSTR = "Gaussian Autoregressive(1)"; 
    static final String HAMSTR = "Hybrid Monte Carlo";
    
    static final double RWMDEFSIGMA = 0.1;
    static final double URWMDEFWIDTHON2 = 0.1;
    static final double LANDEFSIGMA = 0.1;
    static final double LFMDEFSIGMA = 0.1;
    static final double LFMDEFALPHA = 2.0;
    static final double TRWDEFSIGMA = 0.1;
    static final double TRWDEFNU = 1.0;
    static final double GORDEFSIGMA = 0.1;
    static final double GARDEFA = 0.5;
    static final double GARDEFB = 0.5;
    static final double GARDEFSIGMA = 0.1;
    static final double HAMDEFALPHA = 0.5;
    static final double HAMDEFEPS = 0.1;
    static final double HAMDEFL = 1.0;
    
//    GridBagLayout gbl;
//    GridBagConstraints gbc;

    Choice algoChoiceList;
    int algoCard;

    CardLayout algoCards;
    Panel algoCardPanel, rwmCard, urwmCard, lanCard, lfmCard, trwCard,
	uiCard, uuiCard, gorCard, garCard, hamCard;
    ParamInputField rwmParamSigma, urwmParamWidthOn2, lanParamSigma, gorParamSigma,
	lfmParamSigma, lfmParamAlpha, trwParamSigma, trwParamNu,
	garParamA, garParamB, garParamSigma,
	hamParamAlpha, hamParamEps, hamParamL;

    PdfDisplayPanel uuiPdfDisplayPanel;
    Button uuiButtonDraw, uuiButtonAccept, uuiButtonLoad, uuiButtonSave;
    Pdf uuiPdf = null;
    
    public MyObservable gopher;

    AlgoControlPanel() {
	setBackground(Color.white);
	setForeground(Color.black);

//	gbl = new GridBagLayout();
//	gbc = new GridBagConstraints();
	setLayout(new BorderLayout()); //gbl);
	
//	gbc.gridx = 0;
//	gbc.gridy = 0;
//	gbc.weightx = 1.0;
//	gbc.weighty = 1.0;
//	gbc.gridwidth = GridBagConstraints.REMAINDER;
//	gbc.gridheight = 1;
//	gbc.fill = GridBagConstraints.NONE;
	algoChoiceList = new Choice();
//	gbl.setConstraints(algoChoiceList, gbc);
	add("North", algoChoiceList);
	algoChoiceList.addItemListener(this);

//	gbc.gridy = GridBagConstraints.RELATIVE;
//	gbc.gridx = 0;
//	gbc.gridheight = GridBagConstraints.REMAINDER;
	algoCards = new CardLayout();
	algoCardPanel = new Panel(algoCards);
//	gbl.setConstraints(algoCardPanel, gbc);
	add("Center", algoCardPanel);
	
	algoChoiceList.add(RWMSTR);
	rwmCard = new Panel(new GridLayout(2,0));
	rwmCard.add(new Label(
	    "Random Walk Metropolis with Gaussian proposals"));
	rwmParamSigma = new ParamInputField("Sigma: ",
					 ParamInputField.REAL, RWMDEFSIGMA,
					    this);
	rwmCard.add( rwmParamSigma );
	algoCardPanel.add(RWMSTR, rwmCard);

	algoChoiceList.add(URWMSTR);
	urwmCard = new Panel(new GridLayout(2,0));
	urwmCard.add(new Label(
	    "Random Walk Metropolis with uniform proposals"));
	urwmParamWidthOn2 = new ParamInputField("Max jump size (= total width/2): ",
					 ParamInputField.REAL, URWMDEFWIDTHON2,
					    this);
	urwmCard.add( urwmParamWidthOn2 );
	algoCardPanel.add(URWMSTR, urwmCard);

	
	algoChoiceList.add(LANSTR);
 	lanCard = new Panel(new GridLayout(2,0));
 	lanCard.add(new Label("Langevin algorithm"));
 	lanParamSigma = new ParamInputField("Sigma: ",
					     ParamInputField.POSITIVE,
					     LANDEFSIGMA, this);
 	lanCard.add( lanParamSigma ); 
	algoCardPanel.add(LANSTR, lanCard);

	algoChoiceList.add(LFMSTR);
 	lfmCard = new Panel(new GridLayout(3,0));
 	lfmCard.add(new Label("Levy Flight based Metropolis"));
 	lfmParamSigma = new ParamInputField("Sigma: ",
					     ParamInputField.POSITIVE,
					     LFMDEFSIGMA, this);
 	lfmCard.add( lfmParamSigma );
	lfmParamAlpha = new ParamInputField("Alpha in range (0,2]: ",
					    ParamInputField.POSITIVE,
					    LFMDEFALPHA, this);
	lfmCard.add( lfmParamAlpha );
	algoCardPanel.add(LFMSTR, lfmCard);

	algoChoiceList.add(TRWSTR);
 	trwCard = new Panel(new GridLayout(3,0));
 	trwCard.add(new Label("Student t Random Walk"));
 	trwParamSigma = new ParamInputField("Sigma: ",
					     ParamInputField.POSITIVE,
					     TRWDEFSIGMA, this);
 	trwCard.add( trwParamSigma );
	trwParamNu = new ParamInputField("Nu: ",
					    ParamInputField.POSITIVE,
					    TRWDEFNU, this);
	trwCard.add( trwParamNu );
	algoCardPanel.add(TRWSTR, trwCard);

	algoChoiceList.add(UISTR);
	uiCard = new Panel(new GridLayout(2,0));
	uiCard.add(new Label("Independence sampler with U[0,1] proposal"));
	algoCardPanel.add(UISTR, uiCard);

	algoChoiceList.add(UUISTR);
	uuiCard = new Panel(new BorderLayout()); //GridLayout(3,0));
	uuiCard.add("North", new Label(
	    "Independence sampler with user-defined proposal"));
	uuiPdfDisplayPanel = new PdfDisplayPanel();
	uuiCard.add("Center", uuiPdfDisplayPanel);
	uuiButtonDraw = new Button("New");
	uuiButtonDraw.addActionListener(this);
	uuiButtonAccept = new Button("Accept");
	uuiButtonAccept.addActionListener(this);
	uuiButtonLoad = new Button("Load");
	uuiButtonLoad.addActionListener(this);
	uuiButtonSave = new Button("Save");
	uuiButtonSave.addActionListener(this);
	Panel p = new Panel();
	p.add(uuiButtonDraw);
	p.add(uuiButtonAccept);
	p.add(uuiButtonLoad);
	p.add(uuiButtonSave);
	uuiCard.add("South", p);
	algoCardPanel.add(UUISTR, uuiCard);
	
	algoChoiceList.add(GORSTR);
 	gorCard = new Panel(new GridLayout(2,0));
 	gorCard.add(new Label("Gareth's mode hopping algorithm"));
 	gorParamSigma = new ParamInputField("Sigma: ",
					     ParamInputField.POSITIVE,
					     GORDEFSIGMA, this);
 	gorCard.add( gorParamSigma ); 
	algoCardPanel.add(GORSTR, gorCard);
	
	algoChoiceList.add(GARSTR);
	garCard = new Panel(new GridLayout(4,0));
	garCard.add(new Label(
	    "Proposals given by Y = a + b(X - a) + sigma * Z"));
	garParamA = new ParamInputField("a: ",
					 ParamInputField.REAL, GARDEFA,
					    this);
	garParamB = new ParamInputField("b: ",
					 ParamInputField.REAL, GARDEFB,
					    this);
	garParamSigma = new ParamInputField("Sigma: ",
					 ParamInputField.REAL, GARDEFSIGMA,
					    this);
	garCard.add( garParamA );
	garCard.add( garParamB );
	garCard.add( garParamSigma );
	algoCardPanel.add(GARSTR, garCard);

	algoChoiceList.add(HAMSTR);
	hamCard = new Panel(new GridLayout(4,0));
	hamCard.add(new Label(
	    "Radford Neal's Hybrid Monte Carlo"));
	hamParamAlpha = new ParamInputField("alpha: ",
					 ParamInputField.REAL, HAMDEFALPHA,
					    this);
	hamParamEps = new ParamInputField("epsilon: ",
					 ParamInputField.REAL, HAMDEFEPS,
					    this);
	hamParamL = new ParamInputField("Leaps: ",
					ParamInputField.POSITIVE, HAMDEFL,
					this);
	hamCard.add( hamParamAlpha );
	hamCard.add( hamParamEps );
	hamCard.add( hamParamL );
	algoCardPanel.add(HAMSTR, hamCard);

	gopher = new MyObservable();
    }

    public void doInit() {
	algoCard = RWM;
	algoChoiceList.select(RWMSTR);

	gopher.forceChanged();
	gopher.notifyObservers();
    }

    public void addObserver(Observer obj) {
	gopher.addObserver(obj);
    }
    
    public void itemStateChanged(ItemEvent ie) {
	String tmp = ie.getItem().toString();
	Vector gparams = new Vector(4,0);
	
	if( RWMSTR.equals(tmp) ) {
	    gparams.addElement(new Integer(RWM));
	    gparams.addElement(new Double(rwmParamSigma.getValue()));
	    algoCards.show(algoCardPanel, RWMSTR);
	    algoCard = RWM;
	}
	if( URWMSTR.equals(tmp) ) {
	    gparams.addElement(new Integer(URWM));
	    gparams.addElement(new Double(urwmParamWidthOn2.getValue()));
	    algoCards.show(algoCardPanel, URWMSTR);
	    algoCard = URWM;
	}
	else if( LANSTR.equals(tmp) ) {
	    gparams.addElement(new Integer(LAN));
	    gparams.addElement(new Double(lanParamSigma.getValue()));
	    algoCards.show(algoCardPanel, LANSTR);
	    algoCard = LAN;
	}
	else if( LFMSTR.equals(tmp) ) {
	    gparams.addElement(new Integer(LFM));
	    gparams.addElement(new Double(lfmParamSigma.getValue()));
	    gparams.addElement(new Double(lfmParamAlpha.getValue()));
	    algoCards.show(algoCardPanel, LFMSTR);
	    algoCard = LFM;
	}
	else if( TRWSTR.equals(tmp) ) {
	    gparams.addElement(new Integer(TRW));
	    gparams.addElement(new Double(trwParamSigma.getValue()));
	    gparams.addElement(new Double(trwParamNu.getValue()));
	    algoCards.show(algoCardPanel, TRWSTR);
	    algoCard = TRW;
	}
	else if( UISTR.equals(tmp) ) {
	    gparams.addElement(new Integer(UI));
	    algoCards.show(algoCardPanel, UISTR);
	    algoCard = UI;
	}
	else if( UUISTR.equals(tmp) ) {
	    gparams.addElement(new Integer(UUI));
	    gparams.addElement(uuiPdf);
	    algoCards.show(algoCardPanel, UUISTR);
	    algoCard = UUI;
	}
	else if( GORSTR.equals(tmp) ) {
	    gparams.addElement(new Integer(GOR));
	    gparams.addElement(new Double(gorParamSigma.getValue()));
	    algoCards.show(algoCardPanel, GORSTR);
	    algoCard = GOR;
	}
	else if( GARSTR.equals(tmp) ) {
	    gparams.addElement(new Integer(GAR));
	    gparams.addElement(new Double(garParamA.getValue()));
	    gparams.addElement(new Double(garParamB.getValue()));
	    gparams.addElement(new Double(garParamSigma.getValue()));
	    algoCards.show(algoCardPanel, GARSTR);
	    algoCard = GAR;
	}
	else if( HAMSTR.equals(tmp) ) {
	    gparams.addElement(new Integer(HYB));
	    gparams.addElement(new Double(hamParamAlpha.getValue()));
	    gparams.addElement(new Double(hamParamEps.getValue()));
	    Double dl = new Double( hamParamL.getValue());
	    gparams.addElement(new Integer(dl.intValue()));
	    algoCards.show(algoCardPanel, HAMSTR);
	    algoCard = HYB;
	}
	gopher.forceChanged();
	gopher.notifyObservers(gparams);
    }
    
    public void actionPerformed(ActionEvent ae) {
	Vector gparams = new Vector(4,0);
	
	switch(algoCard) {
	    case RWM:
		gparams.addElement(new Integer(RWM));
		gparams.addElement(new Double(rwmParamSigma.getValue()));
		break;
	    case URWM:
		gparams.addElement(new Integer(URWM));
		gparams.addElement(new Double(urwmParamWidthOn2.getValue()));
		break;
	    case LAN:
		gparams.addElement(new Integer(LAN));
		gparams.addElement(new Double(lanParamSigma.getValue()));
		break;
	    case LFM:
		gparams.addElement(new Integer(LFM));
		gparams.addElement(new Double(lfmParamSigma.getValue()));
		gparams.addElement(new Double(lfmParamAlpha.getValue()));
		break;
	    case TRW:
		gparams.addElement(new Integer(TRW));
		gparams.addElement(new Double(trwParamSigma.getValue()));
		gparams.addElement(new Double(trwParamNu.getValue()));
		break;
	    case UI:
		gparams.addElement(new Integer(UI));
		break;
	    case UUI:
		gparams.addElement(new Integer(UUI));
		String str = ae.getActionCommand();
		if( str.equals("New") ) {
		    uuiPdfDisplayPanel.clearDrawingBoard();
		    uuiPdfDisplayPanel.showDrawingBoard();
		    gparams.addElement( null );
		} else if( str.equals("Accept") ) {
		    uuiPdf = uuiPdfDisplayPanel.getPdf();
		    uuiPdfDisplayPanel.setPdf(uuiPdf);
		    uuiPdfDisplayPanel.showGraph();
		    gparams.addElement( uuiPdf );
		} else if( str.equals("Load") ) {
		    gparams.addElement( str );
		    gparams.addElement( uuiPdfDisplayPanel );
		} else if( str.equals("Save") ) {
		    gparams.addElement( str );
		    gparams.addElement( uuiPdfDisplayPanel );
		}
		break;
	    case GOR:
		gparams.addElement(new Integer(GOR));
		gparams.addElement(new Double(gorParamSigma.getValue()));
		break;
	    case GAR:
		gparams.addElement(new Integer(GAR));
		gparams.addElement(new Double(garParamA.getValue()));
		gparams.addElement(new Double(garParamB.getValue()));
		gparams.addElement(new Double(garParamSigma.getValue()));
		break;
	    case HYB:
		gparams.addElement(new Integer(HYB));
		gparams.addElement(new Double(hamParamAlpha.getValue()));
		gparams.addElement(new Double(hamParamEps.getValue()));
		Double dl = new Double( hamParamL.getValue());
		gparams.addElement(new Integer(dl.intValue()));
		break;
	    default:
	}
	gopher.forceChanged();
	gopher.notifyObservers(gparams);
    }
}













