/* 
 * Copyright (C) 1997-2002 Laird Breyer
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Author:   Laird Breyer (laird@lbreyer.com)
 */

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.util.*;
import java.lang.*;

/**
 * This class is designed to function toghether with the field coupler class.
 * @author L.A. Breyer
 * @version 1.0
 */
abstract class CouplableField extends RandomField {
    
	/**
	 * This method returns the value at y of the density (unnormalized)
	 * for the value of the field at x. It should be overriden, as it
	 * returns 0 by default.
	 * @param x location in the interval (0,1)
	 * @param y location for possible value of field X(x)
	 * @return density value at y (unnormalized)
	 * of the random variable X(x)
	 */
    abstract public double kernelDensity(double x, double y);

	/**
	 * This method returns the mean of the (normalized) density
	 * p(x,y) at x. Returns 0.5 by default.
	 * @param x location in the interval (0,1)
	 */
//    abstract public double kernelMean(double x);

	/**
	 * This method returns the variance of the (normalized) density
	 * p(x,y) at x. Returns 0.1 by default.
	 * @param x location in the interval (0,1)
	 */
//    abstract public double kernelVariance(double x);

	/**
	 * Let p(z,y) be the kernel density of the field. The result
	 * of this method is a RealInterval whose endpoints represent
	 * respectively the maximum and minimum values (over y) of
	 * the ratio p(z,y)/p(z0,y), for z belonging to the interval
	 * [z0, z0 + epsilon] (if epsilon > 0) and [z0 + epsilon, z0]
	 * (if epsilon < 0)
	 * @param z0
	 * @param epsilon
	 * @return a RealInterval.
	 */
}

