/* 
 * Copyright (C) 1997-2002 Laird Breyer
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Author:   Laird Breyer (laird@lbreyer.com)
 */

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.util.*;

class DiagControlPanel extends Panel implements ActionListener {

    final static String RESETSTR = "Reset hist.";
    final static String NHISTSTR = "Bins";
    final static String ITDELAYSTR = "Delay";
    final static String ACFSTR = "ACF length";

    final static double DEFNHIST = 50;
    final static double DEFITDELAY = 10;
    final static double DEFACFLAG = 50;
    
    public MyObservable gopher;
    Button resetButton;
    ParamInputField delayParam, acfParam, histParam;
    
    DiagControlPanel() {
	setBackground(Color.white);
	setForeground(Color.black);
	setLayout(new GridLayout(2, 2));

	resetButton = new Button(RESETSTR);
	resetButton.addActionListener(this);
	Panel p = new Panel();
	p.add(resetButton);
	add(p);

	delayParam = new ParamInputField(ITDELAYSTR,
					 ParamInputField.POSITIVE, DEFITDELAY, this);
	add(delayParam);

	histParam = new ParamInputField(NHISTSTR,
					ParamInputField.POSITIVE, DEFNHIST, this);
	add(histParam);
	
	acfParam = new ParamInputField(ACFSTR, ParamInputField.POSITIVE, DEFACFLAG, this);
	add(acfParam);
	
	gopher = new MyObservable();
    }

    public void addObserver(Observer obj) {
	gopher.addObserver(obj);
    }
    
    public void actionPerformed(ActionEvent ae) {
	String str = ae.getActionCommand();

	Vector v = new Vector(2, 0);
	v.addElement(str);
	
	if( str.equals(ITDELAYSTR) ) {
	    long delay = (long)Math.round(delayParam.getValue());
	    v.addElement(new Long(delay));
	} else if( str.equals(NHISTSTR) ) {
	    int nhist = (int)Math.round(histParam.getValue());
	    v.addElement(new Integer(nhist));
	} else {
	    int maxlag = (int)Math.round(acfParam.getValue());
	    v.addElement(new Integer(maxlag));
	}

	gopher.forceChanged();
	gopher.notifyObservers(v);
    }
}
