/* 
 * Copyright (C) 1997-2002 Laird Breyer
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Author:   Laird Breyer (laird@lbreyer.com)
 */

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.util.*;
import java.awt.image.*;

class FieldCouplerControlPanel extends Panel implements ActionListener, ItemListener {

    final static double DEFHISTDELAY = 5;
    final static double DEFITDELAY = 100;
    final static double DEFNUMBINS = 8;
    
    final static String RESETSTR = "reset";

    final static String CB1STR = "Normal histogram";
    final static String CB2STR = "Without diagonal";
    final static String CB3STR = "Superimposed diagonal";
    
    final static String CCB1STR = "view CFTP + MH";
    final static String CCB2STR = "view result + MH";
    final static String CCB3STR = "MH without coupling";
    final static String CCB4STR = "None";
    
    final static String ITDELAY = "delay";
    final static String ITBTHIST = "histogram delay";
    final static String NUMBINS = "bins";
    
    public MyObservable gopher;
    Button resetButton;
    ParamInputField delayParam;
    ParamInputField histDelayParam;
    ParamInputField numbinsParam;

    CheckboxGroup cbg, ccbg;
    Checkbox cb1, cb2, cb3, ccb1, ccb2, ccb3, ccb4 ; 
    
    FieldCouplerControlPanel() {
	setBackground(Color.white);
	setForeground(Color.black);
	setLayout(new FlowLayout());

	resetButton = new Button(RESETSTR);
	resetButton.addActionListener(this);
	add(resetButton);

	cbg = new CheckboxGroup();
	cb1 = new Checkbox(CB1STR, cbg, false);
	cb1.addItemListener(this);
	//add(cb1);
	cb2 = new Checkbox(CB2STR, cbg, false);
	cb2.addItemListener(this);
	//add(cb2);
	cb3 = new Checkbox(CB3STR, cbg, true);	
	cb3.addItemListener(this);
	//add(cb3);

	histDelayParam = new ParamInputField(ITBTHIST,
					     ParamInputField.POSITIVE, DEFHISTDELAY,
					     this);
	//add(histDelayParam);
	
	delayParam = new ParamInputField(ITDELAY,
					 ParamInputField.POSITIVE, DEFITDELAY, this);
	add(delayParam);

	numbinsParam = new ParamInputField(NUMBINS,
					   ParamInputField.POSITIVE, DEFNUMBINS, this);
	add(numbinsParam);
					   
	ccbg = new CheckboxGroup();
	ccb1 = new Checkbox(CCB1STR, ccbg, true);
	ccb1.addItemListener(this);
	add(ccb1);
	ccb2 = new Checkbox(CCB2STR, ccbg, false);
	ccb2.addItemListener(this);
	add(ccb2);
	ccb3 = new Checkbox(CCB3STR, ccbg, false);	
	ccb3.addItemListener(this);
	add(ccb3);
	ccb4 = new Checkbox(CCB4STR, ccbg, false);	
	ccb4.addItemListener(this);
	//add(ccb4);
	
	gopher = new MyObservable();
    }

    public void addObserver(Observer obj) {
	gopher.addObserver(obj);
    }
    
    public void actionPerformed(ActionEvent ae) {
	String str = ae.getActionCommand();	
	if( str.equals(RESETSTR) ) {
	    gopher.forceChanged();
	    gopher.notifyObservers(str);
	} else {
	    gopher.forceChanged();
	    gopher.notifyObservers(ae);
	}
    }

    public void itemStateChanged(ItemEvent ie) {
	gopher.forceChanged();
	gopher.notifyObservers(cbg.getSelectedCheckbox().getLabel());
	
	gopher.forceChanged();
	gopher.notifyObservers(ccbg.getSelectedCheckbox().getLabel());
    }
}



