/* 
 * Copyright (C) 1997-2002 Laird Breyer
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Author:   Laird Breyer (laird@lbreyer.com)
 */

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.util.*;
import java.awt.image.*;

class FieldHistogram extends Canvas {

    int count, numbins, dmax, nmax, gmax, diagType;
    int bins[][], dbins[];
    
    Image img;
    Graphics ig;
    
    Dimension size, remainder, binsize;

    Image sbuf = null;
//    Dimension dold;

    String the_title = null;
    
    FieldHistogram(int n, int t) {
	reset(n, t);
    }

    public void reset(int n, int t) {
	bins = new int[n][n];
	dbins = new int[n];
	numbins = n;
	diagType = t;
	count = 0;
	dmax = 1;
	nmax = 1;
	gmax = 1;

//	dold = new Dimension(0,0);
    }

    public void setType(int t) {
	diagType = t;
    }
    
    public void addField(RandomField x) {
	count++;
	for( int i = 0; i < numbins; i++ ) {
	    double a = (double)i/numbins;
	    double xv = x.valueAt(a);
	    if(xv == a) {
		dbins[i]++;
		dmax = Math.max(dmax, dbins[i]);
		gmax = Math.max(gmax, dbins[i] + bins[i][i]);
	    } else if( (xv >= 0) && (xv < 1) ) {
		int j = (int)(xv * numbins);
		bins[i][j]++;
		nmax = Math.max(nmax, bins[i][j]);
		gmax = Math.max(gmax, bins[i][j]);
	    }
	}
    }

    public void computeHistogram() {

	size = getSize();
	double bw = (double)size.width / numbins;
	double bh = (double)size.height / numbins;
	
//	remainder = new Dimension( size.width % numbins,
//				   size.height % numbins );
	    //	binsize = new Dimension( size.width / numbins, size.height / numbins);

	img = createImage(size.width, size.height); // - remainder.width,
//			  size.height - remainder.height);
	ig = img.getGraphics();

 	for( int i = 0; i < numbins; i++ )
 	    for( int j = 0; j < numbins; j++ ) {
	        float c;

		switch(diagType) {
		    case 1:
			if( i != j )
			    c = (float)bins[i][j]/gmax;
			else
			    c = (float)(bins[i][i] + dbins[i])/gmax;
			break;
		    case 2:
			c = (float)bins[i][j]/(2 * nmax);
			break;
		    case 3:
			if( i != j )
			    c = (float)bins[i][j]/(2 * nmax);
			else
			    c = (float)dbins[i]/(2 * dmax) + 0.5f;
			break;
		    default:
			c = 0;
		}

		float d = (float)Math.sqrt(c);
		ig.setColor(new Color( Color.HSBtoRGB( c, d, d)));
		ig.fillRect((int)(i * bw), (int)((numbins - j) * bh),
			    (int)bw + 1, (int)bh + 1);
	    }
	ig.setColor(Color.white);
	ig.drawString("Field count: " + count, 10, 20);
	if(the_title != null) ig.drawString(the_title, 10, 30);
    }

    public void paint(Graphics g) {
	if( img != null ) {
	    g.drawImage(img, 0, 0, null); //remainder.width/2, remainder.height/2, null);
	} else {
	    Dimension size = getSize();
	    g.setColor(Color.black);
	    g.fillRect(0,0,size.width, size.height);
	}
    }

    public void update(Graphics g) {
	paint(g);
    }

    void set_title(String mytitle) {
	the_title = mytitle;
    }
}

