/* 
 * Copyright (C) 1997-2002 Laird Breyer
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Author:   Laird Breyer (laird@lbreyer.com)
 */

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.util.*;
import java.lang.*;
class GaussianARField extends CouplableField {


    double w, a, b, sigma, sigma2;
    Pdf target;

	/**
	 * This constructor accepts a target density for the drift,
	 * and a magnification factor s for the Gaussian increment.
	 * @param r the random number generator
	 * @param targetDensity a Pdf used to compute the drift
	 * @param s the standard deviation for the Gaussian increment
	 */
    GaussianARField(Random r, double avalue, double bvalue, double s) {
	w = s * r.nextGaussian();
	sigma = s;
	sigma2 = s * s;
	a = avalue;
	b = bvalue;
    }

    GaussianARField respawn(Random r, double avalue, double bvalue, double s) {
	w = s * r.nextGaussian();
	sigma = s;
	sigma2 = s * s;
	a = avalue;
	b = bvalue;
	return this;
    }
    
    public double valueAt(double x) {
	return a + b *(x - a) + w;
    }

    public double kernelDensity(double x, double y) {
	double t = y - a - b * (x - a);
	return Math.exp( - t * t/(2 * sigma2));
    }
    
    public double kernelMean(double x) {
	return a + b *(x - a);
    }

    public double kernelVariance(double x) {
	return  sigma2;
    }

    public double newVariableAt(Random r, double x) {
	double w = sigma * r.nextGaussian();
	return a + b * (x - a) + w;
    }
}
