/* 
 * Copyright (C) 1997-2002 Laird Breyer
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Author:   Laird Breyer (laird@lbreyer.com)
 */

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.util.*;
import java.lang.*;

/**
 * This class implements a random field suitable for using
 * as a proposal with the Levy flight Metropolis sampler, using an alpha
 * stable proposal distribution.
 * @author L.A. Breyer
 * @version 1.0
 */
class LevyFlightField extends CouplableField {

    double w;
    double sigma, sigma2, sigma22, alpha;

	/**
	 * This constructor accepts a value of
	 * s, the magnification factor for the increment, and
	 * a, the alpha stability parameter for the increment.
	 * @param r the random number generator
	 * @param s the standard deviation for the increment
	 * @param a the alpha stability for the increment
	 */
    LevyFlightField(MyRandom r, double s, double a) {
	w = r.nextSymmetricAlpha(a);
	alpha = a;
	sigma = s;
	sigma2 = s * s;
	w = sigma * w;
    }

    LevyFlightField respawn(MyRandom r, double s, double a) {
	w = r.nextSymmetricAlpha(a);
	alpha = a;
	sigma = s;
	sigma2 = s * s;
	w = sigma * w;
	return this;
    }

    public double valueAt(double x) {
	return x + w;
    }

    public double kernelDensity(double x, double y) {
//	double t = (y-x) * (y-x);
//	return Math.exp( - t * t/(2*sigma2) );
	return 0;
    }

    public double kernelMean(double x) {
	return x;
    }

    public double kernelVariance(double x) {
	return sigma2;
    }

    public double newVariableAt(MyRandom r, double x) {
	double w = r.nextSymmetricAlpha(alpha);
	return x + sigma * w;
    }
}
