/* 
 * Copyright (C) 1997-2002 Laird Breyer
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Author:   Laird Breyer (laird@lbreyer.com)
 */

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.util.*;
import java.lang.*;

/**
 * My own random number generator, with bigger choice.
 */
final class MyRandom extends Random {

    final static double ROOT2PI = Math.sqrt( 2 * Math.PI );
    
    	/**
	 * this method generates a symmetric alpha stable random
	 * variable with mean 0 and skewness 0. The parameter alpha
	 * is trusted to be in the range (0, 2]
	 */
    public double nextSymmetricAlpha(double alpha) {
	double V = Math.PI * (nextDouble() - 0.5);
	double k = alpha * V;
	double W = - Math.log(nextDouble())/Math.cos(V-k);
	double h = W * Math.cos(V);
	double s = Math.abs(Math.sin(k) * Math.exp( - Math.log(h)/alpha ) * W);
	s = ( nextDouble() < 0.5 ) ? -s : s;
	return s;
    }

	/**
	 * this method generates a symmetric one dimensional Student t
	 * distribution with parameter nu > 0.
	 */
    public double nextStudent(int nu) {

	double t = 0;
	for(int i = 0; i < nu; i++ ) {
	    double s = nextGaussian();
	    t += s * s;
	}
	t = nextGaussian() / Math.sqrt(t/nu);
	return t;
    }
    
	/**
	 * This method returns a random variable  whose density is equal to
	 * the supremum of all normal densities with variance sigma squared
	 * and mean belonging to the interval (-e, e). In other words, the
	 * density is flat inside the interval (-e,e) and gaussian outside.
	 */
    public double nextPlateauGaussian(double sigma, double e) {
	if( nextDouble() > 2 * e / ( ROOT2PI * sigma + 2 * e ) ) {
	    double w = nextGaussian() * sigma;
	    return ( w > 0 ) ? w + e : w - e;
	}
	else {
	    return 2 * e * nextDouble() - e;
	}
    }
}
