/* 
 * Copyright (C) 1997-2002 Laird Breyer
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Author:   Laird Breyer (laird@lbreyer.com)
 */

import java.awt.*;
import java.awt.event.*;
import java.lang.*;
import java.util.*;
import java.io.*;

class ParamInputField extends Panel implements ActionListener {

    public static final int REAL = 0;
    public static final int POSITIVE = 1;
    
    Label name;
    TextField value;
    double defaultval;
    int inputType;

    ActionListener listener=null;
    
    ParamInputField( String nameStr, int iType, double defaultValue,
		     ActionListener aListener) {
	inputType = iType;
	listener = aListener;
	name = new Label( nameStr, Label.RIGHT);
	value = new TextField(5);
	value.addActionListener(this);

	value.setBackground(Color.white);
	value.setForeground(Color.black);
	
	defaultval = defaultValue;
	value.setText(Double.toString(defaultValue));
	setLayout(new FlowLayout(FlowLayout.LEFT));
	add(name);
	add(value);
    }

    public double getValue() {
	try {
	    return Double.valueOf(value.getText()).doubleValue();
	} catch( NumberFormatException e) {
	    return defaultval;
	}
    }

    public int getIntValue() {
	try {
	    return Integer.valueOf(value.getText()).intValue();
	} catch( NumberFormatException e) {
	    return 0;
	}
    }

    public void actionPerformed(ActionEvent ae) {

	Double dValue;
	try {
	    dValue = Double.valueOf(value.getText());
	    switch(inputType) {
		case POSITIVE:
		    if( dValue.doubleValue() < 0 )
			dValue = new Double(defaultval);
		    break;
		case REAL:
		    break;
		default:
	    }
	} catch(NumberFormatException ne) {
	    dValue = new Double(defaultval);
	}
	switch(inputType) {
	    default:
		value.setText(dValue.toString());
		break;
	}
	

	if(listener != null)
	    listener.actionPerformed(
		new ActionEvent(this, 0, name.getText())); // pass along
    }
}
