/* 
 * Copyright (C) 1997-2002 Laird Breyer
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Author:   Laird Breyer (laird@lbreyer.com)
 */

import java.awt.*;
import java.awt.event.*;
import java.lang.*;
import java.util.*;
import java.io.*;

class PdfDisplayPanel extends Panel {

    CardLayout cl;
    GraphPanel graph;
    PdfDrawPanel draw;
     
    public PdfDisplayPanel() {

	setBackground(Color.black);
	setForeground(Color.black);
	cl = new CardLayout( 1, 1);
	setLayout( cl );

	graph = new GraphPanel();
	draw = new PdfDrawPanel();
	add("graph", graph);
	add("draw", draw);
	cl.show(this, "graph");
    }
    
    void setPdf(Pdf pdf) {
	graph.setFunction(pdf);
//	graph.repaint();
    }

    void clearDrawingBoard() {
	draw.clear(getSize());
    }

    void showDrawingBoard() {
	cl.show(this, "draw");
    }

    void showGraph() {
	cl.show(this, "graph");
    }

    Pdf getPdf() {
	return draw.getPdf();
    }

    public void loadPdf(String where) throws IOException {
	FileReader f = new FileReader(where);
	draw.readShape(f);
	f.close();
    }

    public void savePdf(String where) throws IOException {
	FileWriter f = new FileWriter(where);
	draw.writeShape(f);
	f.close();
    }

    public void repaint() {
	graph.repaint();
    }
}
