/* 
 * Copyright (C) 1997-2002 Laird Breyer
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Author:   Laird Breyer (laird@lbreyer.com)
 */

import java.awt.*;
import java.awt.event.*;
import java.lang.*;
import java.util.*;
import java.io.*;

class PdfDrawPanel extends Panel implements MouseListener, MouseMotionListener {
    Vector lines;

    public PdfDrawPanel() {
        setBackground(Color.orange);
	setForeground(Color.black);

	addMouseMotionListener(this);
        addMouseListener(this);

	lines= new Vector();
	clear( getSize() );
    }

    Pdf getPdf() {
 	Rectangle r;
	double shapeX[];
	double shapeY[];
 	int n, h, w;
 	double min;
	
 	w = ((Rectangle)lines.lastElement()).width;
	n = lines.size();
	if( (w <= 0) || (n==1) )
	    return (Pdf)null;
 	
 	h = getSize().height;
 	min = 0.0;

	shapeX = new double[n+1];
	shapeY = new double[n+1];

	shapeX[0] = 0.0;
	shapeY[0] = 0.0;
	r = (Rectangle)(lines.elementAt(0));
	int j = 1;
	for( int i=1; i < n; i++) {
  	    r = (Rectangle)(lines.elementAt(i));
	    if( r.x != r.width ) {
		shapeX[j] =  (double)r.x / (double) w;
		shapeY[j] = h - (double)r.y;
		min = (min < shapeY[j]) ? min : shapeY[j];
		j++;
	    }
  	}

	n = j;
	shapeX[n] = r.width / (double) w;
	shapeY[n] = h - (double)r.height;
// 	shapeX[n] = ((Rectangle)lines.elementAt(n-1)).width / (double) w;
// 	shapeY[n] = h - ((Rectangle)lines.elementAt(n-1)).height;
 	min = (min < shapeY[n]) ? min : shapeY[n];

 	n++;
 	min = min - 1;
 	for( int i= 0; i < n; i++)
 	    shapeY[i] -= min;
	
 	return new PolygonalPdf(n, shapeX, shapeY);
    }

    public void writeShape(FileWriter w) throws IOException {
	int n = lines.size();
	String str = new String("\'Pdf\'");

	w.write(str, 0, str.length());

	for( Enumeration e = lines.elements(); e.hasMoreElements(); ) {
	    Rectangle r = (Rectangle)e.nextElement();
	    str = " " + r.width + " " + r.height;
	    w.write(str, 0, str.length());
	}
    }
    
    public void readShape(FileReader r) throws IOException {
	StreamTokenizer tok = new StreamTokenizer(r);
	boolean error = false;
	boolean found = false;

	int xold = 0, yold = 0;

	while( tok.nextToken() != tok.TT_EOF ) {

//	    if( error )
//		break;
	    
 	    if( !found )
 		if( (tok.ttype == '\'') && tok.sval.equals("Pdf") ) {
 		    found = true;
 		    lines.removeAllElements();
 		} else {
 		    error = true;
 		    break;
 		}
	    
	    switch(tok.ttype) {
		case StreamTokenizer.TT_NUMBER:
		    int x = (int)tok.nval;
		    if( (tok.nextToken() == tok.TT_EOF) || (tok.ttype != tok.TT_NUMBER) ) 
			error = true;
		    
		    int y = (int)tok.nval;
		    lines.addElement(new Rectangle(xold, yold, x, y));
		    xold = x;
		    yold = y;
		    break;
		default:
//		    error = true;
	    }
	    repaint();
	}
	
	if( error ) {
	    
	    repaint();
		// throw new IOException();
}	else
	    repaint();
    }
    
    public void mouseDragged(MouseEvent e) {
	mousePressed(e);
    }

    public void mouseMoved(MouseEvent e) {
    }

    public void clear(Dimension d) {
//	Dimension d2 = getSize();
	lines.removeAllElements();
	lines.addElement(new Rectangle(0, d.height,
				       0, d.height));
    }

    public void mousePressed(MouseEvent e) {
	e.consume();
	
	Rectangle last;
	int i = lines.size();

	do {
	    last = (Rectangle)lines.elementAt(i-1);
	    if( last.width > e.getX() )
		lines.removeElementAt(i-1);
	    else
		break;
	    i--;
	} while( i > 1 );

	int mousex = Math.max(e.getX(), 0);
	if( lines.size() == 0 )
	    clear(getSize());
	
	last = (Rectangle)lines.lastElement();	
	lines.addElement(new Rectangle(last.width, last.height,
					   mousex, e.getY()));
	
	repaint();   
    }

    public void mouseReleased(MouseEvent e) {
	e.consume();
	repaint();  
    }
    
    public void mouseEntered(MouseEvent e) {
    }

    public void mouseExited(MouseEvent e) {
    }

    public void mouseClicked(MouseEvent e) {
    }   

    public void paint(Graphics g) {

        int np = lines.size();

	g.drawString( "points: " + np, 10, 10);

	g.setPaintMode();
        for (int i=1; i < np; i++) {
            Rectangle p = (Rectangle)lines.elementAt(i);
	    g.drawLine(p.x, p.y, p.width, p.height);
	}
    } 
}
