/* 
 * Copyright (C) 1997-2002 Laird Breyer
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Author:   Laird Breyer (laird@lbreyer.com)
 */


/**
 * This class implements a subinterval of the real line, to be used by other
 * classes using this package.
 * @author L.A. Breyer
 * @version 1.0
 * @param beginning a double representing the beginning of the interval.
 * @param end a double representing the end of the interval.
 */
final public class RealInterval {
    
    public double beginning;
    public double end;

	/**
	 * This is the default constructor which creates the interval (0,1)
	 */
    public RealInterval() {
	beginning = 0;
	end = 1;
    }

	/**
	 * This constructor creates an arbitrary interval.
	 * @param b the beginning of the interval.
	 * @param e the end of the interval.
	 */
    public RealInterval(double b, double e) {
	beginning = b;
	end = e;
    }

    public double midpoint() {
	return (beginning + end)/2;
    }    

    public double length() {
	return end - beginning;
    }
}


