/* 
 * Copyright (C) 1997-2002 Laird Breyer
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Author:   Laird Breyer (laird@lbreyer.com)
 */

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.util.*;
import java.io.*;

public class RoadMapFrame extends Frame implements Runnable, Observer, WindowListener {

    MyRandom r;
    Pdf pdf, uuiPdf;
    int algo;
    CouplableField x = null, gx = null;
    RandomField rx;
    double pdfMax;
        
    double rwmSigma;
    double urwmWidthOn2;
    double lanSigma;
    double gorSigma;
    double garA, garB, garSigma;
    double lfmSigma, lfmAlpha;
    double trwSigma, trwNu;

    double h, Mag, z, y, yold, oldval, newval, oldtargetval, newtargetval;
    String msg;

    Thread appletThread = null;
    long delay;
    int numbins;
    boolean bins[];
    long histDelay;
    int histcount, histType = 3, histGridSize;
    int demoType = 1;
    boolean resetFlag = false, graphoff = false;
    
    PdfDisplayPanel pdfDisplayPanel;
    PdfControlPanel pdfControlPanel;
    
    CoupledFieldGraphPanel coupledFieldGraphPanel;
    AlgoControlPanel algoControlPanel;

    FieldHistogram fieldHistogram;
    TimeHistogram timeHistogram;
    RoadMapControlPanel rmControlPanel;

    FieldDensityGraph fieldDensityGraph;
    
    Font f;
    static boolean standalone = false;

    RoadMapFrame() {
	super("MH Roadmap");
	addWindowListener(this);

	setFont(new Font("Dialog",Font.PLAIN, 10));

	setLayout(new GridLayout(0,2,0,0));
	
	Panel p1 = new Panel(new GridLayout(3,0));
	Panel p2 = new Panel(new GridLayout(3,0));

	pdfDisplayPanel = new PdfDisplayPanel();
	Panel temp = new Panel(new GridLayout(2,0));
	temp.add(new TitlePanel("MH Roadmap"));
	temp.add(pdfDisplayPanel);
	p1.add(temp);

	pdfControlPanel = new PdfControlPanel();
	pdfControlPanel.addObserver(this);
	p2.add(pdfControlPanel);

	coupledFieldGraphPanel = new CoupledFieldGraphPanel();
	p1.add(coupledFieldGraphPanel);

 	algoControlPanel = new AlgoControlPanel();
	algoControlPanel.addObserver(this);
	p2.add(algoControlPanel);

	histGridSize = (int)RoadMapControlPanel.DEFNUMBINS;
	fieldHistogram = new FieldHistogram(histGridSize, histType);
	p1.add(fieldHistogram);

	rmControlPanel = new RoadMapControlPanel();
	rmControlPanel.addObserver(this);
	p2.add(rmControlPanel);

	add(p1);
	add(p2);
	
	r = new MyRandom();
    }

    public void init() {
	delay = (long)RoadMapControlPanel.DEFITDELAY;
	numbins = 8;
	histDelay = (long)RoadMapControlPanel.DEFHISTDELAY;
	histcount = 1;
//	resetFlag = true;
	pdf = new Normal(PdfControlPanel.NORMALDEFMU, PdfControlPanel.NORMALDEFSIGMA2);
	//	pdf = new Trimodal(PdfControlPanel.TRIDEFSIGMA,PdfControlPanel.TRIDEFA,PdfControlPanel.TRIDEFB,PdfControlPanel.TRIDEFC);
	
	pdfMax = 1;
	histType = 3;
	
	pdfDisplayPanel.setPdf(pdf);
	pdfDisplayPanel.clearDrawingBoard();
	pdfDisplayPanel.showGraph();
	
	algo = AlgoControlPanel.RWM;
	rwmSigma = AlgoControlPanel.RWMDEFSIGMA;
	
	pdfDisplayPanel.clearDrawingBoard();
	fieldHistogram.reset(histGridSize, histType);
	fieldHistogram.repaint();

    }
    
    public void update(Observable obj, Object arg) {
	if(obj == pdfControlPanel.gopher) {
	    if( arg == null ) {
		pdf = null;
		pdfDisplayPanel.setPdf(pdf);	
		pdfDisplayPanel.showGraph();
	    } else if( arg instanceof String ) {
		if( ((String)arg).equals("New") ) {
		    pdf = null;
		    pdfDisplayPanel.clearDrawingBoard();
		    pdfDisplayPanel.showDrawingBoard();
		} else if( ((String)arg).equals("Accept") ) {
		    pdf = pdfDisplayPanel.getPdf();
		    pdfDisplayPanel.setPdf(pdf);
		    pdfDisplayPanel.showGraph();
		} else if( ((String)arg).equals("Load") ) {
		    FileDialog fd = new FileDialog(this, "Load a pdf", FileDialog.LOAD);
		    fd.setFile("*.pdf");
		    fd.setVisible(true);
		    if( fd.getFile() != null ) {
			String where = fd.getDirectory() + fd.getFile();
			try {
			    pdfDisplayPanel.loadPdf(where);
			    pdf = pdfDisplayPanel.getPdf();
			    pdfDisplayPanel.setPdf(pdf);
			} catch( IOException ioe ) {
			    pdf = null;
			}
		    }
		    fd = null;
		} else if( ((String)arg).equals("Save") ) {
		    FileDialog fd = new FileDialog(this, "Save a pdf", FileDialog.SAVE);
		    fd.setFile("noname.pdf");
		    fd.setVisible(true);
		    if( fd.getFile() != null ) {
			String where = fd.getDirectory() + fd.getFile();
			try {
			    pdfDisplayPanel.savePdf(where);
			} catch( IOException ioe ) {
				// do nothing
			}
		    }			
		    fd = null;
		}
	    } else if(arg instanceof Pdf) {
		pdf = (Pdf)arg;
		pdfDisplayPanel.setPdf(pdf);
		pdfDisplayPanel.showGraph();		
	    }
	    
	    if( pdf != null )
		pdfMax = pdf.getDensityRange().end;
	    else
		pdfMax = 1;

	    pdfDisplayPanel.repaint();
	    fieldHistogram.reset(histGridSize, histType);
	    fieldHistogram.repaint();

	    histcount = 1;
//	    resetFlag = true;		    
	} else if( obj == algoControlPanel.gopher ) {
	    gx = null;
	    x = null;
	    if(arg == null) {
		algo = AlgoControlPanel.RWM;
		rwmSigma = AlgoControlPanel.RWMDEFSIGMA;
	    } else {
		algo = ((Integer)
			((Vector)arg).elementAt(0)).intValue();
		switch(algo) {
		    case AlgoControlPanel.RWM:
			rwmSigma = ((Double)
				    ((Vector)arg).elementAt(1)).doubleValue();
			break;
		    case AlgoControlPanel.URWM:
			urwmWidthOn2 = ((Double)
				    ((Vector)arg).elementAt(1)).doubleValue();
			break;
		    case AlgoControlPanel.LAN:
			lanSigma = ((Double)
				    ((Vector)arg).elementAt(1)).doubleValue();
			break;
		    case AlgoControlPanel.UI:
			uuiPdf = null;
			break;
		    case AlgoControlPanel.UUI:
			Object e = ((Vector)arg).elementAt(1);
			if( e == null ) {
			    uuiPdf = null;
			} else if( e instanceof Pdf ) {
			    uuiPdf = (Pdf) e;
			} else if( e instanceof String ) {
			    PdfDisplayPanel udp = (PdfDisplayPanel)((Vector)arg).elementAt(2);
			    if( ((String)e).equals("Load") ) {
				FileDialog fd = new FileDialog(this, "Load a proposal pdf",
							       FileDialog.LOAD);
				fd.setFile("*.pdf");				
				fd.setVisible(true);
				if( fd.getFile() != null ) {
				    String where = fd.getDirectory() + fd.getFile();
				    try {
					udp.loadPdf(where);
					uuiPdf = udp.getPdf();
					udp.setPdf(uuiPdf);
				    } catch( IOException ioe ) {
					uuiPdf = null;
				    }
				}
				fd = null;
				udp.repaint();
			    } else if( ((String)e).equals("Save")) {
				FileDialog fd = new FileDialog(this, "Save a pdf", FileDialog.SAVE);
				fd.setFile("noname.pdf");
				fd.setVisible(true);
				if( fd.getFile() != null ) {
				    String where = fd.getDirectory() + fd.getFile();
				    try {
					udp.savePdf(where);
				    } catch( IOException ioe ) {
				// do nothing
				    }
				}			
				fd = null;
				udp.repaint();
			    }
			}
			break;
		    case AlgoControlPanel.GOR:
			gorSigma = ((Double)
				    ((Vector)arg).elementAt(1)).doubleValue();
			break;
		    case AlgoControlPanel.GAR:
			garA = ((Double)
				    ((Vector)arg).elementAt(1)).doubleValue();
			garB = ((Double)
				    ((Vector)arg).elementAt(2)).doubleValue();
			garSigma = ((Double)
				    ((Vector)arg).elementAt(3)).doubleValue();
			break;
		    default:			
		}		
	    }
	    fieldHistogram.reset(histGridSize, histType);	    
	    fieldHistogram.repaint();

	    histcount = 1;
//	    resetFlag = true;
	    
	} else if( obj == rmControlPanel.gopher ) {
	    if( arg instanceof String ) {
		if( ((String)arg).equals(RoadMapControlPanel.RESETSTR)) {
		    fieldHistogram.reset(histGridSize, histType);
		    fieldHistogram.repaint();
		    histcount = 1;
		    resetFlag = true;
		} else if( ((String)arg).equals(RoadMapControlPanel.CB1STR)) {
		    histType = 1;
		    fieldHistogram.reset(histGridSize, histType);
		    fieldHistogram.repaint();
		    histcount = 1;
		} else if( ((String)arg).equals(RoadMapControlPanel.CB2STR)) {
		    histType = 3;
		    fieldHistogram.reset(histGridSize, histType);
		    fieldHistogram.repaint();
		    histcount = 1;
		} else if( ((String)arg).equals("graphon")) {
		    graphoff = false;
		} else if( ((String)arg).equals("graphoff")) {
//			RoadMapControlPanel.CB3STR)) {
		    graphoff = true;
//		    histGridSize = 50;
		}
	    } else if( arg instanceof ActionEvent ) {
		double dvalue = ((ParamInputField)
				 ((ActionEvent)arg).getSource()).getValue();
		String ac = ((ActionEvent)arg).getActionCommand();
		if( ac.equals(RoadMapControlPanel.ITDELAY) ) 
		    delay = (long)Math.round(dvalue);
		else if( ac.equals(RoadMapControlPanel.ITBTHIST) ) 
		    histDelay = (long)Math.round(dvalue);
		else {
		    histGridSize = (int)Math.round(dvalue);
		    fieldHistogram.reset(histGridSize, histType);
		    fieldHistogram.repaint();
		    histcount = 1;
		}
	    }
	}
    }

    public void start() {
	if( pdf == null ) {
		// do nothing
	} else {
	    if( appletThread == null ) {
		appletThread = new Thread(this);
	    }
	    appletThread.start();	
	    //	    } else if( !appletThread.isAlive() )
	    //		appletThread.start();	    
	}
    }

    public void stop() {
	appletThread = null;
    }
     
    public void run() {
	Thread.currentThread().setPriority(Thread.MIN_PRIORITY);
	Thread currentThread = Thread.currentThread();
	
	while( currentThread == appletThread ) {
	    try {
		Thread.sleep(delay);
		Thread.sleep(10); //for netscape
	    } catch( InterruptedException e) {};

	    if( pdf == null ) 	
		continue;

	    switch(algo) {
		case AlgoControlPanel.RWM:
		    if( gx == null )
			gx = new GaussianRandomWalkField(r, rwmSigma);
		    else
			gx = ((GaussianRandomWalkField) gx).respawn(r, rwmSigma);
		    if( histType != 1 )
			if( rx == null )
			    rx = new SymPropHMField( r, pdf, gx);
			else
			    rx = ((SymPropHMField)rx).respawn2( r, pdf, gx);
		    break;
		case AlgoControlPanel.URWM:
		    if( gx == null )
			gx = new UniformRandomWalkField(r, urwmWidthOn2);
		    else
			gx = ((UniformRandomWalkField) gx).respawn(r, urwmWidthOn2);
		    if( histType != 1 )
			if( rx == null )
			    rx = new SymPropHMField( r, pdf, gx);
			else
			    rx = ((SymPropHMField)rx).respawn2( r, pdf, gx);
		    break;
		case AlgoControlPanel.LAN:
		    if( gx  == null)			
			gx = new LangevinField(r, pdf, lanSigma);
		    else
			gx = ((LangevinField) gx).respawn(r, pdf, lanSigma);
		    if( histType != 1 )
			if( rx == null )
			    rx = new HMField( r, pdf, gx);
			else
			    rx = ((HMField)rx).respawn( r, pdf, gx);
		    break;
		case AlgoControlPanel.GOR:
		    if( gx == null )
			gx = new GorField(r, pdf, gorSigma);
		    else
			gx = ((GorField) gx).respawn(r, pdf, gorSigma);
		    if( histType != 1 )
			if( rx == null )
			    rx = new HMField( r, pdf, gx);
			else
			    rx = ((HMField)rx).respawn( r, pdf, gx);
		    break;
		case AlgoControlPanel.LFM:
		    if( gx == null )
			gx = new LevyFlightField(r, lfmSigma, lfmAlpha);
		    else
			gx = ((LevyFlightField)gx).respawn(r, lfmSigma, lfmAlpha);
		    if( histType != 1 )
			if( rx == null )
			    rx = new SymPropHMField( r, pdf, gx);
			else
			    rx = ((SymPropHMField)rx).respawn2( r, pdf, gx);
		    break;
		case AlgoControlPanel.TRW: 
		    if( gx == null )
			gx = new StudentRandomWalkField(r, trwSigma, (int)trwNu);
		    else
			gx = ((StudentRandomWalkField)gx).respawn(r, trwSigma, (int)trwNu);
		    if( histType != 1 )
			if( rx == null)
			    rx = new SymPropHMField( r, pdf, gx);
			else
			    rx = ((SymPropHMField)rx).respawn2( r, pdf, gx);
		    break;
		case AlgoControlPanel.UI: 
		    if( gx == null )
			gx = new IndependenceField(r);
		    else
			gx = ((IndependenceField)gx).respawn(r);
		    if( histType != 1 )
			if( rx == null)
			    rx = new HMField( r, pdf, gx);
			else
			    rx = ((HMField)rx).respawn( r, pdf, gx);
		    break;
		case AlgoControlPanel.UUI:
		    if( uuiPdf == null )
			continue;
		    else {
			if( gx == null)
			    gx = new IndependenceField(r, uuiPdf);
			else
			    gx = ((IndependenceField)gx).respawn(r, uuiPdf);
		    }
		    if( histType != 1 )
			if( rx== null)
			    rx = new HMField( r, pdf, gx);
			else
			    rx = ((HMField)rx).respawn( r, pdf, gx);
		    break;
		case AlgoControlPanel.GAR:
		    if( gx == null )
			gx = new GaussianARField(r, garA, garB, garSigma);
		    else
			gx = ((GaussianARField)gx).respawn(r, garA, garB, garSigma);
		    if( histType != 1 )
			if( rx == null )
			    rx = new HMField( r, pdf, gx);
			else
			    rx = ((HMField)rx).respawn( r, pdf, gx);
		    break;
		case AlgoControlPanel.HYB: // dummy
		default:
		    gx = null;
		    rx = null;
	    }

	    if( histType == 1 ) {
		if( !graphoff )
		    coupledFieldGraphPanel.setField(gx);
		fieldHistogram.addField(gx);
	    } else {
//		rx = new HMField( r, pdf, gx);
		if( !graphoff )
		    coupledFieldGraphPanel.setField(rx);
		fieldHistogram.addField(rx);
	    }
	    if( !graphoff )
		coupledFieldGraphPanel.repaint();
	    
 	    histcount++;
 	    if( histcount > histDelay) {
 		histcount = 1;
 		fieldHistogram.computeHistogram();
 		fieldHistogram.repaint();		
 	    }
	}
    }

    public void windowActivated(WindowEvent we) {
    }

    public void windowDeactivated(WindowEvent we) {
    }

    public void windowClosed(WindowEvent we) {
    }

    public void windowClosing(WindowEvent we) {
	stop();
	setVisible(false);
	if(standalone)
	    System.exit(0);
    }

    public void windowDeiconified(WindowEvent we) {
    }

    public void windowIconified(WindowEvent we) {
    }

    public void windowOpened(WindowEvent we) {	
    }

    public static void main(String args[]) {

	standalone = true;
	
	RoadMapFrame c = new RoadMapFrame();

	//c.pack(); 
	c.setTitle("MH Roadmap");
	c.setSize(600,450);
	c.setVisible(true);
	c.init();
	c.start();
    }
}    











