/* 
 * Copyright (C) 1997-2002 Laird Breyer
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Author:   Laird Breyer (laird@lbreyer.com)
 */

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.util.*;
import java.io.*;

class SimAFrame2 extends Frame implements Runnable, Observer, WindowListener {

    MyRandom r;
    Pdf pdf, hotPdf;
    int algo, cooling;
    RandomField x;
    CouplableField cx;
    HybridMonteCarloField hx;
    double chain, hybridChain[], pdfMax;
    int n; // time
    double C, alpha, beta, gamma;
    
    double rwmSigma;
    double lanSigma;
    double gorSigma;
    double garA, garB, garSigma;
    double lfmSigma, lfmAlpha;
    double trwSigma, trwNu;
    double hamAlpha, hamEps;
    int hamL;
    Pdf uuiPdf;

    double h, Mag, z, y, yold, oldval, newval, oldtargetval, newtargetval;
    String msg;
    Thread appletThread = null;
    long delay;
    int acfMaxLag, histNBins;

    PdfDisplayPanel pdfDisplayPanel;
    PdfControlPanel pdfControlPanel;
    
    ChainTimeSeries chainTimeSeries;
    AlgoControlPanel algoControlPanel;

    TargetAverage targetAverage;
    DiagControlPanel histControlPanel;

    CoolingControlPanel coolingControlPanel;
    
    Font f;
    static boolean standalone = false;
    
    SimAFrame2() {
	super("Simulated Annealing Algorithms II");
	addWindowListener(this);

	setLayout(new GridLayout(0,2, 0, 0)); 

	Panel p1 = new Panel(new GridLayout(3,0));
	Panel p2 = new Panel(new GridLayout(3,0));
	
	pdfDisplayPanel = new PdfDisplayPanel();
	Panel temp = new Panel(new GridLayout(2,0));
	temp.add(new TitlePanel("Simulated Annealing Algorithms II"));
	temp.add(pdfDisplayPanel);
	p1.add(temp);

	pdfControlPanel = new PdfControlPanel();
	pdfControlPanel.addObserver(this);
	p2.add(pdfControlPanel);
	
	chainTimeSeries = new ChainTimeSeries(50);
	chainTimeSeries.addObserver(this);
	p1.add(chainTimeSeries);

 	algoControlPanel = new AlgoControlPanel();
	algoControlPanel.addObserver(this);
	p2.add(algoControlPanel);

	targetAverage = new TargetAverage();
	p1.add(targetAverage);

	histControlPanel = new DiagControlPanel();
	histControlPanel.addObserver(this);
	Panel p = new Panel(new GridLayout(2, 0));
	p.add(histControlPanel);

	coolingControlPanel = new CoolingControlPanel();
	coolingControlPanel.addObserver(this);
	p.add(coolingControlPanel);

	p2.add(p);	

	add(p1);
	add(p2);
	
	r = new MyRandom();
    }

    public void init() {
	
	chain = 0.5;
	hybridChain = new double[2];
	hybridChain[0] = 0.5;
	hybridChain[1] = 1;
	
	delay = (long)DiagControlPanel.DEFITDELAY;
	acfMaxLag = (int)DiagControlPanel.DEFACFLAG;
	histNBins = (int)DiagControlPanel.DEFNHIST;
	
	pdf = new Normal(PdfControlPanel.NORMALDEFMU, PdfControlPanel.NORMALDEFSIGMA2);
	pdfMax = 1;
	
	pdfDisplayPanel.setPdf(pdf);
	pdfDisplayPanel.clearDrawingBoard();
	pdfDisplayPanel.showGraph();

	targetAverage.reset(histNBins);
	targetAverage.calibrate(pdf);

	algo = AlgoControlPanel.RWM;
	rwmSigma = AlgoControlPanel.RWMDEFSIGMA;

	cooling = CoolingControlPanel.LOG;
	C = 1.0;
	alpha = 1.0;
	n = 0;

    }
    
    public void update(Observable obj, Object arg) {
	if(obj == pdfControlPanel.gopher) {
	    if( arg == null ) {
		pdf = null;
		pdfDisplayPanel.setPdf(pdf);
		pdfDisplayPanel.showGraph();
	    } else {
		if( arg instanceof String ) {
		    if( ((String)arg).equals("New") ) {
			pdfDisplayPanel.clearDrawingBoard();
			pdfDisplayPanel.showDrawingBoard();
			pdf = null;
		    } else if( ((String)arg).equals("Accept") ) {
			pdf = pdfDisplayPanel.getPdf();
			pdfDisplayPanel.setPdf(pdf);
			pdfDisplayPanel.showGraph();
			targetAverage.reset(histNBins);
			targetAverage.calibrate(pdf);

		    } else if( ((String)arg).equals("Load") ) {
			FileDialog fd = new FileDialog(this, "Load a pdf", FileDialog.LOAD);
			fd.setFile("*.pdf");
			fd.setVisible(true);
			if( fd.getFile() != null ) {
			    String where = fd.getDirectory() + fd.getFile();
			    try {
				pdfDisplayPanel.loadPdf(where);
 				pdf = pdfDisplayPanel.getPdf();
 				pdfDisplayPanel.setPdf(pdf);
 				targetAverage.reset(histNBins);
 				targetAverage.calibrate(pdf);

			    } catch( IOException ioe ) {
				pdf = null;
			    }
			}
			fd = null;
		    } else if( ((String)arg).equals("Save") ) {
			FileDialog fd = new FileDialog(this, "Save a pdf", FileDialog.SAVE);
			fd.setFile("noname.pdf");
			fd.setVisible(true);
			if( fd.getFile() != null ) {
			    String where = fd.getDirectory() + fd.getFile();
			    try {
			    pdfDisplayPanel.savePdf(where);
			    } catch( IOException ioe ) {
				// do nothing
			    }
			}			
			fd = null;
		    }
		} else if( arg instanceof Pdf ) {
		    pdf = (Pdf)arg;
		    pdfDisplayPanel.setPdf(pdf);
		    pdfDisplayPanel.showGraph();
		    targetAverage.reset(histNBins);
		    targetAverage.calibrate(pdf);

		}
		if( pdf != null )
		    pdfMax = pdf.getDensityRange().end;
		else
		    pdfMax = 1;
	    }
	    n = 0; // reset time
	} else if( obj == algoControlPanel.gopher ) {
	    if(arg == null) {
		algo = AlgoControlPanel.RWM;
		rwmSigma = AlgoControlPanel.RWMDEFSIGMA;
	    } else {
		algo = ((Integer)
			((Vector)arg).elementAt(0)).intValue();
		switch(algo) {
		    case AlgoControlPanel.RWM:
			rwmSigma = ((Double)
				    ((Vector)arg).elementAt(1)).doubleValue();
			break;
		    case AlgoControlPanel.LAN:
			lanSigma = ((Double)
				    ((Vector)arg).elementAt(1)).doubleValue();
			break;
		    case AlgoControlPanel.LFM:
			lfmSigma = ((Double)
				    ((Vector)arg).elementAt(1)).doubleValue();
			lfmAlpha = ((Double)
				    ((Vector)arg).elementAt(2)).doubleValue();
			break;
		    case AlgoControlPanel.TRW:
			trwSigma = ((Double)
				    ((Vector)arg).elementAt(1)).doubleValue();
			trwNu = ((Double)
				    ((Vector)arg).elementAt(2)).doubleValue();
			break;
		    case AlgoControlPanel.UI:
			uuiPdf = null;
			break;
		    case AlgoControlPanel.UUI:
			Object e = ((Vector)arg).elementAt(1);
			if( e == null ) {
			    uuiPdf = null;
			} else if( e instanceof Pdf ) {
			    uuiPdf = (Pdf) e;
			} else if( e instanceof String ) {
			    PdfDisplayPanel udp = (PdfDisplayPanel)((Vector)arg).elementAt(2);
			    if( ((String)e).equals("Load") ) {
				FileDialog fd = new FileDialog(this, "Load a proposal pdf",
							       FileDialog.LOAD);
				fd.setFile("*.pdf");				
				fd.setVisible(true);
				if( fd.getFile() != null ) {
				    String where = fd.getDirectory() + fd.getFile();
				    try {
					udp.loadPdf(where);
					uuiPdf = udp.getPdf();
					udp.setPdf(uuiPdf);
				    } catch( IOException ioe ) {
					uuiPdf = null;
				    }
				}
				fd = null;
				udp.repaint();
			    } else if( ((String)e).equals("Save")) {
				FileDialog fd = new FileDialog(this, "Save a pdf", FileDialog.SAVE);
				fd.setFile("noname.pdf");
				fd.setVisible(true);
				if( fd.getFile() != null ) {
				    String where = fd.getDirectory() + fd.getFile();
				    try {
					udp.savePdf(where);
				    } catch( IOException ioe ) {
				// do nothing
				    }
				}			
				fd = null;
				udp.repaint();
			    }
			}
			break;
		    case AlgoControlPanel.GOR:
			gorSigma = ((Double)
				    ((Vector)arg).elementAt(1)).doubleValue();
			break;
		    case AlgoControlPanel.GAR:
			garA = ((Double)
				((Vector)arg).elementAt(1)).doubleValue();
			garB = ((Double)
				((Vector)arg).elementAt(2)).doubleValue();
			garSigma = ((Double)
				    ((Vector)arg).elementAt(3)).doubleValue();
			break;
		    case AlgoControlPanel.HYB:
			hamAlpha = ((Double)
				    ((Vector)arg).elementAt(1)).doubleValue();
			hamEps = ((Double)
				  ((Vector)arg).elementAt(2)).doubleValue();
			hamL = ((Integer)
				((Vector)arg).elementAt(3)).intValue();
			break;
		    default:
		}
	    }
	    n = 0; // reset time
	} else if( obj == histControlPanel.gopher ) {
	    String str = (String)((Vector)arg).elementAt(0);
	    if( str.equals(DiagControlPanel.RESETSTR) ) {
		targetAverage.reset(histNBins);
		targetAverage.calibrate(pdf);
		n = 0; // reset time
	    }
	    else if( str.equals(DiagControlPanel.NHISTSTR) ) {
		histNBins = ((Integer)((Vector)arg).elementAt(1)).intValue();
		targetAverage.reset(histNBins);
		targetAverage.calibrate(pdf);
		n = 0; // reset time
	    }
	    else if( str.equals(DiagControlPanel.ITDELAYSTR) ) {
		delay = ((Long)((Vector)arg).elementAt(1)).longValue();
	    }
	} else if( obj == chainTimeSeries.gopher ) {
	    chain = ((Double)arg).doubleValue();
	} else if( obj == coolingControlPanel.gopher ) {
	    if(arg == null) {
		cooling = CoolingControlPanel.LOG;
		C = CoolingControlPanel.LOGDEFC;
	    } else {
		cooling = ((Integer)
			((Vector)arg).elementAt(0)).intValue();
		switch(cooling) {
		    case CoolingControlPanel.LOG:
			C = ((Double)
				    ((Vector)arg).elementAt(1)).doubleValue();
			break;
		    case CoolingControlPanel.LIN:
			C = ((Double)
				    ((Vector)arg).elementAt(1)).doubleValue();
			break;
		    case CoolingControlPanel.POW:
			C = ((Double)
				    ((Vector)arg).elementAt(1)).doubleValue();
			alpha = ((Double)
				    ((Vector)arg).elementAt(2)).doubleValue();
			break;
		    case CoolingControlPanel.FRAC:
			C = ((Double)
				    ((Vector)arg).elementAt(1)).doubleValue();
			alpha = ((Double)
				    ((Vector)arg).elementAt(2)).doubleValue();
			gamma = ((Double)
				    ((Vector)arg).elementAt(3)).doubleValue();
			break;
		    default:
		}
	    }
	    n = 0; // reset time    
	}
	targetAverage.repaint();

	System.gc();	
    }

    public void start() {
	if( pdf == null ) {
		// do nothing
	} else {
	    if( appletThread == null ) {
		appletThread = new Thread(this);
		appletThread.start();	
	    } else if( !appletThread.isAlive() )
		appletThread.start();	    
	}
    }

    public void stop() {
	appletThread = null;
    }
    
    public void run() {
	Thread.currentThread().setPriority(Thread.MIN_PRIORITY);
	Thread currentThread = Thread.currentThread();
	
	while( currentThread == appletThread ) {
	    try {
		Thread.sleep(delay);
		Thread.sleep(5); //for netscape
	    } catch( InterruptedException e) {};

	    if( pdf == null ) 	
		continue;

	    n++;
	    switch(cooling) {
		case CoolingControlPanel.LOG:
		    beta = Math.log(n)/C;
		    break;
		case CoolingControlPanel.POW:
		    beta = Math.pow(alpha, -n)/C;
		    break;
		case CoolingControlPanel.FRAC:
		    beta = Math.pow(alpha, -Math.pow(n, gamma))/C;
		    break;
		case CoolingControlPanel.LIN:
		    beta = C * n;
		default:
		    beta = Math.log(n)/C;		    
	    }

	    hotPdf = new HotPdf(beta, pdf);
	    
	    switch(algo) {
		case AlgoControlPanel.RWM: 
		    if( r.nextDouble() > 0.5 ) {
			cx = new GaussianRandomWalkField(r, rwmSigma/beta);
			x = new SymPropHMField(r, pdf, cx);
			chain = x.hotValueAt(chain, beta);
		    } else {
			cx = new GaussianRandomWalkField(r, rwmSigma);
			x = new SymPropHMField(r, pdf, cx);
			chain = x.hotValueAt(chain, beta);
		    }
		    break;
		case AlgoControlPanel.LAN: 
		    cx = new LangevinField(r, hotPdf, lanSigma);
		    x = new HMField(r, pdf, cx);
		    chain = x.hotValueAt(chain, beta);
		    break;
		case AlgoControlPanel.LFM: 
		    cx = new LevyFlightField(r, lfmSigma, lfmAlpha);
		    x = new SymPropHMField(r, pdf, cx);
		    chain = x.hotValueAt(chain, beta);
		    break;
		case AlgoControlPanel.TRW: 
		    cx = new StudentRandomWalkField(r, trwSigma, (int)trwNu);
		    x = new SymPropHMField(r, pdf, cx);
		    chain = x.hotValueAt(chain, beta);
		    break;
		case AlgoControlPanel.UI: 
		    cx = new IndependenceField(r);
		    x = new HMField(r, pdf, cx);			    
		    chain = x.hotValueAt(chain, beta);
		    break;
		case AlgoControlPanel.UUI:
		    if( uuiPdf == null )
			continue;
		    else {
			cx = new IndependenceField(r, uuiPdf);
			x = new HMField(r, pdf, cx);
			chain = x.hotValueAt(chain, beta);
		    }
		    break;
		case AlgoControlPanel.GOR:
		    cx = new GorField(r, hotPdf, gorSigma);
		    x = new HMField(r, pdf, cx);
		    chain = x.hotValueAt(chain, beta);
		    break;
		case AlgoControlPanel.GAR:
		    cx = new GaussianARField(r, garA, garB, garSigma);
		    x = new HMField(r, pdf, cx);
		    chain = x.hotValueAt(chain, beta);
		    break;
		case AlgoControlPanel.HYB:
		    hx = new HybridMonteCarloField(r, hamAlpha, hamEps, hamL,
						   hotPdf);
		    hybridChain = hx.valueAt(hybridChain);
		    chain = hybridChain[0];
		    double kinetic = hybridChain[1] * hybridChain[1] / 2;
		    double potential = - beta * Math.log(pdf.valueAt(chain)/pdfMax);
		    chainTimeSeries.setEnergy(kinetic, potential);
		default:
	    }
	    
	    chainTimeSeries.addPoint(chain);		
	    targetAverage.addPoint(chain);

	    pdfDisplayPanel.setPdf(hotPdf);	    

	    pdfDisplayPanel.repaint();
	    chainTimeSeries.repaint();
	    targetAverage.repaint();
	}
    }

    public void windowActivated(WindowEvent we) {
    }

    public void windowDeactivated(WindowEvent we) {
    }

    public void windowClosed(WindowEvent we) {
    }

    public void windowClosing(WindowEvent we) {
	stop();
	setVisible(false);
	if(standalone)
	    System.exit(0);
    }

    public void windowDeiconified(WindowEvent we) {
    }

    public void windowIconified(WindowEvent we) {
    }

    public void windowOpened(WindowEvent we) {	
    }

    public static void main(String args[]) {

	standalone = true;
	
	SimAFrame2 c = new SimAFrame2();

	c.pack(); 
	c.setTitle("Simulated Annealing Algorithms");
	c.setVisible(true);
	c.init();
	c.start();
    }
}    














